@@ -47,6 +47,7 @@
 #include "ED_view3d.hh"
 
 #include "UI_interface.hh"
+#include "UI_interface_layout.hh"
 #include "UI_resources.hh"
 
 #include "paint_intern.hh" /* own include */
@@ -257,7 +258,7 @@ static wmOperatorStatus weight_sample_invoke(bContext *C, wmOperator *op, const
       MEM_SAFE_FREE(defbase_unlocked);
 
       CLAMP(vgroup_weight, 0.0f, 1.0f);
-      BKE_brush_weight_set(vc.scene, brush, vgroup_weight);
+      BKE_brush_weight_set(&ts->wpaint->paint, brush, vgroup_weight);
       changed = true;
     }
   }
@@ -365,14 +366,15 @@ static wmOperatorStatus weight_sample_group_invoke(bContext *C,
       C, WM_operatortype_name(op->type, op->ptr).c_str(), ICON_NONE);
   uiLayout *layout = UI_popup_menu_layout(pup);
   wmOperatorType *ot = WM_operatortype_find("OBJECT_OT_vertex_group_set_active", false);
-  wmOperatorCallContext opcontext = WM_OP_EXEC_DEFAULT;
-  uiLayoutSetOperatorContext(layout, opcontext);
+  blender::wm::OpCallContext opcontext = blender::wm::OpCallContext::ExecDefault;
+  layout->operator_context_set(opcontext);
   int i = 0;
   LISTBASE_FOREACH_INDEX (bDeformGroup *, dg, &mesh->vertex_group_names, i) {
     if (groups[i] == false) {
       continue;
     }
-    PointerRNA op_ptr = layout->op(ot, dg->name, ICON_NONE, WM_OP_EXEC_DEFAULT, UI_ITEM_NONE);
+    PointerRNA op_ptr = layout->op(
+        ot, dg->name, ICON_NONE, blender::wm::OpCallContext::ExecDefault, UI_ITEM_NONE);
     RNA_property_enum_set(&op_ptr, ot->prop, i);
   }
   UI_popup_menu_end(C, pup);
@@ -493,11 +495,10 @@ static bool weight_paint_set(Object *ob, float paintweight)
 
 static wmOperatorStatus weight_paint_set_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
   Object *obact = CTX_data_active_object(C);
   ToolSettings *ts = CTX_data_tool_settings(C);
   Brush *brush = BKE_paint_brush(&ts->wpaint->paint);
-  float vgroup_weight = BKE_brush_weight_get(scene, brush);
+  float vgroup_weight = BKE_brush_weight_get(&ts->wpaint->paint, brush);
 
   if (ED_wpaint_ensure_data(C, op->reports, WPAINT_ENSURE_MIRROR, nullptr) == false) {
     return OPERATOR_CANCELLED;
@@ -594,10 +595,9 @@ static void gradientVert_update(WPGradient_userData *grad_data, int index)
     BLI_assert(grad_data->type == WPAINT_GRADIENT_TYPE_RADIAL);
     alpha = len_v2v2(grad_data->sco_start, vs->sco) * grad_data->sco_line_div;
   }
-  /* no need to clamp 'alpha' yet */
 
   /* adjust weight */
-  alpha = BKE_brush_curve_strength_clamped(grad_data->brush, alpha, 1.0f);
+  alpha = BKE_brush_curve_strength_clamped(grad_data->brush, std::max(0.0f, alpha), 1.0f);
 
   if (alpha != 0.0f) {
     MDeformVert *dv = &grad_data->dvert[index];
@@ -818,10 +818,10 @@ static wmOperatorStatus paint_weight_gradient_exec(bContext *C, wmOperator *op)
     VPaint *wp = ts->wpaint;
     Brush *brush = BKE_paint_brush(&wp->paint);
 
-    BKE_curvemapping_init(brush->curve);
+    BKE_curvemapping_init(brush->curve_distance_falloff);
 
     data.brush = brush;
-    data.weightpaint = BKE_brush_weight_get(scene, brush);
+    data.weightpaint = BKE_brush_weight_get(&wp->paint, brush);
     data.use_vgroup_restrict = (ts->wpaint->flag & VP_FLAG_VGROUP_RESTRICT) != 0;
   }
 
@@ -851,20 +851,23 @@ static wmOperatorStatus paint_weight_gradient_exec(bContext *C, wmOperator *op)
   if (scene->toolsettings->auto_normalize) {
     const int vgroup_num = BLI_listbase_count(&mesh->vertex_group_names);
     bool *lock_flags = BKE_object_defgroup_lock_flags_get(ob, vgroup_num);
+    if (!lock_flags) {
+      lock_flags = MEM_malloc_arrayN<bool>(vgroup_num, "lock_flags");
+      std::memset(lock_flags, 0, vgroup_num); /* Clear to false. */
+      lock_flags[data.def_nr] = true;
+    }
     bool *vgroup_validmap = BKE_object_defgroup_validmap_get(ob, vgroup_num);
     if (vgroup_validmap != nullptr) {
       MDeformVert *dvert = dverts;
+      Span<bool> subset_flags_span = Span(vgroup_validmap, vgroup_num);
+      Span<bool> lock_flags_span = Span(lock_flags, vgroup_num);
+
       for (int i = 0; i < mesh->verts_num; i++) {
         if ((data.vert_cache->elem[i].flag & WPGradient_vertStore::VGRAD_STORE_IS_MODIFIED) != 0) {
-          if (lock_flags != nullptr) {
-            BKE_defvert_normalize_lock_map(
-                &dvert[i], vgroup_validmap, vgroup_num, lock_flags, vgroup_num);
-          }
-          else {
-            BKE_defvert_normalize_lock_single(&dvert[i], vgroup_validmap, vgroup_num, data.def_nr);
-          }
+          BKE_defvert_normalize_lock_map(dvert[i], subset_flags_span, lock_flags_span);
         }
       }
+      MEM_SAFE_FREE(lock_flags);
       MEM_freeN(vgroup_validmap);
     }
   }
