@@ -17,7 +17,7 @@
 #include "DNA_view2d_types.h"
 
 #include "BLI_rect.h"
-#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 #include "BLI_threads.h"
 #include "BLI_utildefines.h"
 
@@ -60,7 +60,7 @@ static void draw_render_info(
   Render *re = RE_GetSceneRender(scene);
   Scene *stats_scene = ED_render_job_get_scene(C);
   if (stats_scene == nullptr) {
-    stats_scene = CTX_data_scene(C);
+    stats_scene = scene;
   }
 
   RenderResult *rr = BKE_image_acquire_renderresult(stats_scene, ima);
@@ -86,7 +86,7 @@ static void draw_render_info(
       GPU_matrix_scale_2f(zoomx, zoomy);
 
       uint pos = GPU_vertformat_attr_add(
-          immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+          immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
       immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
       immUniformThemeColor(TH_FACE_SELECT);
 
@@ -139,7 +139,8 @@ void ED_image_draw_info(Scene *scene,
 
   GPU_blend(GPU_BLEND_ALPHA);
 
-  uint pos = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
 
   /* noisy, high contrast make impossible to read if lower alpha is used. */
@@ -153,17 +154,17 @@ void ED_image_draw_info(Scene *scene,
   BLF_size(blf_mono_font, 11.0f * UI_SCALE_FAC);
 
   BLF_color3ub(blf_mono_font, 255, 255, 255);
-  SNPRINTF(str, "X:%-4d  Y:%-4d |", x, y);
+  SNPRINTF_UTF8(str, "X:%-4d  Y:%-4d |", x, y);
   BLF_position(blf_mono_font, dx, dy, 0);
   BLF_draw(blf_mono_font, str, sizeof(str));
   dx += BLF_width(blf_mono_font, str, sizeof(str));
 
   if (channels == 1 && (cp != nullptr || fp != nullptr)) {
     if (fp != nullptr) {
-      SNPRINTF(str, " Val:%-.3f |", fp[0]);
+      SNPRINTF_UTF8(str, " Val:%-.3f |", fp[0]);
     }
     else if (cp != nullptr) {
-      SNPRINTF(str, " Val:%-.3f |", cp[0] / 255.0f);
+      SNPRINTF_UTF8(str, " Val:%-.3f |", cp[0] / 255.0f);
     }
     BLF_color3ub(blf_mono_font, 255, 255, 255);
     BLF_position(blf_mono_font, dx, dy, 0);
@@ -174,13 +175,13 @@ void ED_image_draw_info(Scene *scene,
   if (channels >= 3) {
     BLF_color3ubv(blf_mono_font, red);
     if (fp) {
-      SNPRINTF(str, "  R:%-.5f", fp[0]);
+      SNPRINTF_UTF8(str, "  R:%-.5f", fp[0]);
     }
     else if (cp) {
-      SNPRINTF(str, "  R:%-3d", cp[0]);
+      SNPRINTF_UTF8(str, "  R:%-3d", cp[0]);
     }
     else {
-      STRNCPY(str, "  R:-");
+      STRNCPY_UTF8(str, "  R:-");
     }
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
@@ -188,13 +189,13 @@ void ED_image_draw_info(Scene *scene,
 
     BLF_color3ubv(blf_mono_font, green);
     if (fp) {
-      SNPRINTF(str, "  G:%-.5f", fp[1]);
+      SNPRINTF_UTF8(str, "  G:%-.5f", fp[1]);
     }
     else if (cp) {
-      SNPRINTF(str, "  G:%-3d", cp[1]);
+      SNPRINTF_UTF8(str, "  G:%-3d", cp[1]);
     }
     else {
-      STRNCPY(str, "  G:-");
+      STRNCPY_UTF8(str, "  G:-");
     }
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
@@ -202,13 +203,13 @@ void ED_image_draw_info(Scene *scene,
 
     BLF_color3ubv(blf_mono_font, blue);
     if (fp) {
-      SNPRINTF(str, "  B:%-.5f", fp[2]);
+      SNPRINTF_UTF8(str, "  B:%-.5f", fp[2]);
     }
     else if (cp) {
-      SNPRINTF(str, "  B:%-3d", cp[2]);
+      SNPRINTF_UTF8(str, "  B:%-3d", cp[2]);
     }
     else {
-      STRNCPY(str, "  B:-");
+      STRNCPY_UTF8(str, "  B:-");
     }
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
@@ -217,13 +218,13 @@ void ED_image_draw_info(Scene *scene,
     if (channels == 4) {
       BLF_color3ub(blf_mono_font, 255, 255, 255);
       if (fp) {
-        SNPRINTF(str, "  A:%-.4f", fp[3]);
+        SNPRINTF_UTF8(str, "  A:%-.4f", fp[3]);
       }
       else if (cp) {
-        SNPRINTF(str, "  A:%-3d", cp[3]);
+        SNPRINTF_UTF8(str, "  A:%-3d", cp[3]);
       }
       else {
-        STRNCPY(str, "- ");
+        STRNCPY_UTF8(str, "- ");
       }
       BLF_position(blf_mono_font, dx, dy, 0);
       BLF_draw(blf_mono_font, str, sizeof(str));
@@ -241,16 +242,14 @@ void ED_image_draw_info(Scene *scene,
         rgba[3] = linearcol[3];
       }
 
-      if (use_default_view) {
-        IMB_colormanagement_pixel_to_display_space_v4(
-            rgba, rgba, nullptr, &scene->display_settings);
-      }
-      else {
-        IMB_colormanagement_pixel_to_display_space_v4(
-            rgba, rgba, &scene->view_settings, &scene->display_settings);
-      }
+      IMB_colormanagement_pixel_to_display_space_v4(rgba,
+                                                    rgba,
+                                                    (use_default_view) ? nullptr :
+                                                                         &scene->view_settings,
+                                                    &scene->display_settings,
+                                                    DISPLAY_SPACE_COLOR_INSPECTION);
 
-      SNPRINTF(str, "  |  CM  R:%-.4f  G:%-.4f  B:%-.4f", rgba[0], rgba[1], rgba[2]);
+      SNPRINTF_UTF8(str, "  |  Display  R:%-.4f  G:%-.4f  B:%-.4f", rgba[0], rgba[1], rgba[2]);
       BLF_position(blf_mono_font, dx, dy, 0);
       BLF_draw(blf_mono_font, str, sizeof(str));
       dx += BLF_width(blf_mono_font, str, sizeof(str));
@@ -283,14 +282,11 @@ void ED_image_draw_info(Scene *scene,
   }
 
   if (color_manage) {
-    if (use_default_view) {
-      IMB_colormanagement_pixel_to_display_space_v4(
-          finalcol, col, nullptr, &scene->display_settings);
-    }
-    else {
-      IMB_colormanagement_pixel_to_display_space_v4(
-          finalcol, col, &scene->view_settings, &scene->display_settings);
-    }
+    IMB_colormanagement_pixel_to_display_space_v4(finalcol,
+                                                  col,
+                                                  (use_default_view) ? nullptr :
+                                                                       &scene->view_settings,
+                                                  &scene->display_settings);
   }
   else {
     copy_v4_v4(finalcol, col);
@@ -306,7 +302,8 @@ void ED_image_draw_info(Scene *scene,
                 ymin + 0.85f * UI_UNIT_Y);
 
   /* BLF uses immediate mode too, so we must reset our vertex format */
-  pos = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  pos = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
 
   if (channels == 4) {
@@ -349,7 +346,8 @@ void ED_image_draw_info(Scene *scene,
   immUnbindProgram();
 
   /* draw outline */
-  pos = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  pos = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   immUniformColor3ub(128, 128, 128);
   imm_draw_box_wire_2d(pos, color_rect.xmin, color_rect.ymin, color_rect.xmax, color_rect.ymax);
@@ -375,12 +373,12 @@ void ED_image_draw_info(Scene *scene,
                  BLI_YUV_ITU_BT709);
     }
 
-    SNPRINTF(str, "V:%-.4f", val);
+    SNPRINTF_UTF8(str, "V:%-.4f", val);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
     dx += BLF_width(blf_mono_font, str, sizeof(str));
 
-    SNPRINTF(str, "   L:%-.4f", lum);
+    SNPRINTF_UTF8(str, "   L:%-.4f", lum);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
   }
@@ -388,22 +386,22 @@ void ED_image_draw_info(Scene *scene,
     rgb_to_hsv(finalcol[0], finalcol[1], finalcol[2], &hue, &sat, &val);
     rgb_to_yuv(finalcol[0], finalcol[1], finalcol[2], &lum, &u, &v, BLI_YUV_ITU_BT709);
 
-    SNPRINTF(str, "H:%-.4f", hue);
+    SNPRINTF_UTF8(str, "H:%-.4f", hue);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
     dx += BLF_width(blf_mono_font, str, sizeof(str));
 
-    SNPRINTF(str, "  S:%-.4f", sat);
+    SNPRINTF_UTF8(str, "  S:%-.4f", sat);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
     dx += BLF_width(blf_mono_font, str, sizeof(str));
 
-    SNPRINTF(str, "  V:%-.4f", val);
+    SNPRINTF_UTF8(str, "  V:%-.4f", val);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
     dx += BLF_width(blf_mono_font, str, sizeof(str));
 
-    SNPRINTF(str, "   L:%-.4f", lum);
+    SNPRINTF_UTF8(str, "   L:%-.4f", lum);
     BLF_position(blf_mono_font, dx, dy, 0);
     BLF_draw(blf_mono_font, str, sizeof(str));
   }
@@ -415,7 +413,7 @@ void draw_image_sample_line(SpaceImage *sima)
 
     GPUVertFormat *format = immVertexFormat();
     uint shdr_dashed_pos = GPU_vertformat_attr_add(
-        format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+        format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
     immBindBuiltinProgram(GPU_SHADER_3D_LINE_DASHED_UNIFORM_COLOR);
 
@@ -451,6 +449,14 @@ void draw_image_main_helpers(const bContext *C, ARegion *region)
     ED_space_image_get_zoom(sima, region, &zoomx, &zoomy);
     draw_render_info(C, sima->iuser.scene, ima, region, zoomx, zoomy);
   }
+
+  if (sima->mode == SI_MODE_UV) {
+    const Scene *scene = CTX_data_scene(C);
+    const ToolSettings *ts = scene->toolsettings;
+    if (ts->uv_flag & UV_FLAG_CUSTOM_REGION) {
+      draw_image_uv_custom_region(region, ts->uv_custom_region);
+    }
+  }
 }
 
 bool ED_space_image_show_cache(const SpaceImage *sima)
@@ -525,7 +531,8 @@ void draw_image_cache(const bContext *C, ARegion *region)
   /* Draw current frame. */
   x = (cfra - sfra) / (efra - sfra + 1) * region->winx;
 
-  uint pos = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   immUniformThemeColor(TH_CFRAME);
   immRectf(pos, x, region_bottom, x + ceilf(framelen), region_bottom + 8 * UI_SCALE_FAC);
@@ -607,3 +614,30 @@ float ED_space_image_increment_snap_value(const int grid_dimensions,
   /* Fallback */
   return grid_steps[0];
 }
+
+void draw_image_uv_custom_region(const ARegion *region, const rctf &custom_region)
+{
+  const uint shdr_pos = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
+
+  GPU_line_width(1.0f);
+
+  immBindBuiltinProgram(GPU_SHADER_3D_LINE_DASHED_UNIFORM_COLOR);
+
+  float viewport_size[4];
+  GPU_viewport_size_get_f(viewport_size);
+  immUniform2f("viewport_size", viewport_size[2] / UI_SCALE_FAC, viewport_size[3] / UI_SCALE_FAC);
+
+  immUniform1i("colors_len", 0); /* "simple" mode */
+  immUniform4f("color", 1.0f, 0.25f, 0.25f, 1.0f);
+  immUniform1f("dash_width", 6.0f);
+  immUniform1f("udash_factor", 0.5f);
+  rcti region_rect;
+
+  UI_view2d_view_to_region_rcti(&region->v2d, &custom_region, &region_rect);
+
+  imm_draw_box_wire_2d(
+      shdr_pos, region_rect.xmin, region_rect.ymin, region_rect.xmax, region_rect.ymax);
+
+  immUnbindProgram();
+}
