@@ -10,7 +10,7 @@
 #include "mtl_debug.hh"
 #include "mtl_framebuffer.hh"
 
-#include "intern/GHOST_ContextCGL.hh"
+#include "intern/GHOST_ContextMTL.hh"
 
 #include <fstream>
 
@@ -51,7 +51,7 @@ id<MTLCommandBuffer> MTLCommandBufferManager::ensure_begin()
      * NOTE: We currently stall until completion of GPU work upon ::submit if we have reached the
      * in-flight command buffer limit. */
     BLI_assert(MTLCommandBufferManager::num_active_cmd_bufs_in_system <
-               GHOST_ContextCGL::max_command_buffer_count);
+               GHOST_ContextMTL::max_command_buffer_count);
 
     if (G.debug & G_DEBUG_GPU) {
       /* Debug: Enable Advanced Errors for GPU work execution. */
@@ -138,12 +138,12 @@ bool MTLCommandBufferManager::submit(bool wait)
   /* If we have too many active command buffers in flight, wait until completed to avoid running
    * out. We can increase */
   if (MTLCommandBufferManager::num_active_cmd_bufs_in_system >=
-      (GHOST_ContextCGL::max_command_buffer_count - 1))
+      (GHOST_ContextMTL::max_command_buffer_count - 1))
   {
     wait = true;
     MTL_LOG_WARNING(
         "Maximum number of command buffers in flight. Host will wait until GPU work has "
-        "completed. Consider increasing GHOST_ContextCGL::max_command_buffer_count or reducing "
+        "completed. Consider increasing GHOST_ContextMTL::max_command_buffer_count or reducing "
         "work fragmentation to better utilize system hardware. Command buffers are flushed upon "
         "GPUContext switches, this is the most common cause of excessive command buffer "
         "generation.");
@@ -576,9 +576,9 @@ void MTLCommandBufferManager::unfold_pending_debug_groups()
 }
 
 /* Workload Synchronization. */
-bool MTLCommandBufferManager::insert_memory_barrier(eGPUBarrier barrier_bits,
-                                                    eGPUStageBarrierBits before_stages,
-                                                    eGPUStageBarrierBits after_stages)
+bool MTLCommandBufferManager::insert_memory_barrier(GPUBarrier barrier_bits,
+                                                    GPUStageBarrierBits before_stages,
+                                                    GPUStageBarrierBits after_stages)
 {
   /* Apple Silicon does not support memory barriers for RenderCommandEncoder's.
    * We do not currently need these due to implicit API guarantees. However, render->render
