@@ -1,4 +1,4 @@
-/* SPDX-FileCopyrightText: 2023 Blender Foundation
+/* SPDX-FileCopyrightText: 2023 Blender Authors
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
@@ -10,9 +10,11 @@ namespace blender::nodes::node_fn_invert_rotation_cc {
 
 static void node_declare(NodeDeclarationBuilder &b)
 {
+  b.use_custom_socket_order();
+  b.allow_any_socket_order();
   b.is_function_node();
   b.add_input<decl::Rotation>("Rotation");
-  b.add_output<decl::Rotation>("Rotation");
+  b.add_output<decl::Rotation>("Rotation").align_with_previous();
 };
 
 static void node_build_multi_function(NodeMultiFunctionBuilder &builder)
@@ -24,11 +26,15 @@ static void node_build_multi_function(NodeMultiFunctionBuilder &builder)
 
 static void node_register()
 {
-  static bNodeType ntype;
-  fn_node_type_base(&ntype, FN_NODE_INVERT_ROTATION, "Invert Rotation", NODE_CLASS_CONVERTER);
+  static blender::bke::bNodeType ntype;
+  fn_node_type_base(&ntype, "FunctionNodeInvertRotation", FN_NODE_INVERT_ROTATION);
+  ntype.ui_name = "Invert Rotation";
+  ntype.ui_description = "Compute the inverse of the given rotation";
+  ntype.enum_name_legacy = "INVERT_ROTATION";
+  ntype.nclass = NODE_CLASS_CONVERTER;
   ntype.declare = node_declare;
   ntype.build_multi_function = node_build_multi_function;
-  nodeRegisterType(&ntype);
+  blender::bke::node_register_type(ntype);
 }
 NOD_REGISTER_NODE(node_register)
 
